package osscli

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"mime/multipart"
	"net/http"
	"os"
	"strings"
	"time"
)

type IOssBucket interface {
	UploadFile(fpath string) (string, error)
}

type ossBucket struct {
	host string
}

func NewOssBucket(host string) IOssBucket {
	return &ossBucket{
		host: host,
	}
}

func (b *ossBucket) UploadFile(filePath string) (string, error) {
	// 检查文件是否存在
	_, err := os.Stat(filePath)
	if err != nil {
		return "", err
	}
	os.Getenv("ALIBABA_CLOUD_ACCESS_KEY_ID")
	// 打开本地文件
	file, err := os.Open(filePath)
	if err != nil {
		return "", err
	}
	defer file.Close()

	// 创建HTTP POST请求
	var reqBody bytes.Buffer
	writer := multipart.NewWriter(&reqBody)
	destFile, err := writer.CreateFormFile("file", file.Name())
	if err != nil {
		return "", err
	}
	if _, err = io.Copy(destFile, file); err != nil {
		return "", err
	}
	_ = writer.Close()

	request, err := http.NewRequest("POST", b.host+"/oss/upload/storage", &reqBody)
	if err != nil {
		return "", err
	}

	// 设置必要的请求头，特别是 Content-Type
	request.Header.Set("Content-Type", writer.FormDataContentType())

	// 设置 businessCode 请求头
	request.Header.Set("businessCode", "4001001")

	// 设置超时时间
	client := &http.Client{
		Timeout: 30 * time.Second,
	}

	// 发送HTTP POST请求
	resp, err := client.Do(request)
	if err != nil {
		return "", err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {

		return "", fmt.Errorf("oss上传响应异常：%d", resp.StatusCode)
	}

	var response map[string]interface{}
	if err := json.NewDecoder(resp.Body).Decode(&response); err != nil {
		return "", err
	}
	if int(response["code"].(float64)) != 200 {
		return "", fmt.Errorf("oss上传失败: code=%d, msg=%s", response["code"].(float64), response["msg"].(string))
	}

	return response["data"].(string), nil
}

type BucketMap string

const (
	GalaxyImmiCdn        BucketMap = `galaxy-immi-cdn`
	GalaxyImmiMp         BucketMap = `galaxy-immi-mp`
	GalaxyImmi           BucketMap = `galaxy-immi`
	GalaxyHkJoin         BucketMap = `galaxy-hkjoin`
	GalaxyDwp            BucketMap = `galaxy-dwp`
	GalaxyFileCvt        BucketMap = `galaxy-filecvt`
	YinHeTest            BucketMap = `yinhe-test`
	GalaxyFile           BucketMap = `galaxy-file`
	GalaxySmartingBucket BucketMap = `galaxy-smartwriting`
	GalaxyDwp2Bucket     BucketMap = `galaxy-dwp2`
	TestBucketV1         BucketMap = `test-bucket-v1`
	TestBucketV1pub      BucketMap = `test-bucket-v1-pub`
	GalaxyDiy            BucketMap = `galaxy-diy`
	HkOss                BucketMap = `test-hk-bucket`
	HkOssProd            BucketMap = `prod-hk-bucket`
	ProdBucketOss        BucketMap = `prod-bucket-v1`
	ProdPubBucketOss     BucketMap = `prod-bucket-v1-pub`
)

func GetBucketMaps() []BucketMap {
	return []BucketMap{
		GalaxyImmiCdn, GalaxyImmiMp, GalaxyImmi, GalaxyHkJoin, GalaxyDwp, GalaxyFileCvt, YinHeTest, GalaxyFile, GalaxyDwp2Bucket, GalaxySmartingBucket, TestBucketV1, TestBucketV1pub, GalaxyDiy, HkOss, HkOssProd, ProdPubBucketOss, ProdBucketOss,
	}
}

func ParseBucketUrl(objectUrl string) (string, string, string) {
	newUrl := strings.ReplaceAll(strings.ReplaceAll(objectUrl, `https://`, ``), `http://`, ``)
	objectBucket := strings.Split(newUrl, `.`)[0]

	var realObjectUrl, realBucket, endPoint string

	buckets := GetBucketMaps()
	for _, bucket := range buckets {
		for _, host := range bucket.Hosts() {
			if strings.Contains(objectUrl, host) || bucket.String() == objectBucket {
				var result []string
				if strings.Contains(objectUrl, `oss-cn-shenzhen.aliyuncs.com`) {
					result = strings.Split(objectUrl, `oss-cn-shenzhen.aliyuncs.com`)
				} else {
					result = strings.Split(objectUrl, host)
				}
				if len(result) != 2 {
					continue
				}
				realObjectUrl = strings.TrimLeft(result[1], `/`)
				realBucket = bucket.String()
				endPoint = host
				goto End
			}
		}
	}
End:
	index := strings.Index(realObjectUrl, "?")
	if index >= 0 {
		realObjectUrl = realObjectUrl[:index]
	}
	return realBucket, realObjectUrl, endPoint
}

func (b BucketMap) Hosts() []string {
	switch b {
	case TestBucketV1:
		return []string{`test-cdn.galaxy-immi.com`}
	case TestBucketV1pub:
		return []string{`test-cdn-pub.galaxy-immi.com`}
	case GalaxyImmiCdn:
		return []string{`cdnfile.galaxy-immi.com`}
	case GalaxyImmiMp:
		return []string{`upload.cdn.galaxy-immi.com`, `upload-cdn.galaxy-immi.com`}
	case GalaxyImmi:
		return []string{`cache.galaxy-immi.com`}
	case GalaxyHkJoin:
		return []string{`galaxy-hkjoin-cdn.galaxy-immi.com`}
	case GalaxyDwp:
		return []string{`galaxy-dwp-cdn.galaxy-immi.com`}
	case GalaxyFileCvt:
		return []string{`galaxy-immi-mp.oss-cn-shenzhen.aliyuncs.com`}
	case YinHeTest:
		return []string{`yinhe-test.oss-cn-shenzhen.aliyuncs.com`}
	case GalaxyFile:
		return []string{`file.galaxy-immi.com`}
	case GalaxySmartingBucket:
		return []string{`galaxy-smartwriting.oss-cn-shenzhen.aliyuncs.com`}
	case GalaxyDwp2Bucket:
		return []string{`galaxy-dwp2.oss-cn-shenzhen.aliyuncs.com`}
	case GalaxyDiy:
		return []string{`galaxy-diy.oss-cn-shenzhen.aliyuncs.com`}
	case HkOss:
		return []string{`test-hk-bucket.oss-cn-hongkong.aliyuncs.com`}
	case HkOssProd:
		return []string{`prod-hk-bucket.oss-cn-hongkong.aliyuncs.com`}
	case ProdBucketOss:
		return []string{`prod-cdn.galaxy-immi.com`}
	case ProdPubBucketOss:
		return []string{`prod-cdn-pub.galaxy-immi.com`}
	default:
		return nil
	}
}

func (b BucketMap) String() string {
	return string(b)
}
